@@ -3,17 +3,20 @@ import logging
 handler = logging.StreamHandler()
 logging.getLogger("ossapi").addHandler(handler)
 
-from ossapi.ossapi import (Ossapi, ReplayUnavailableException,
+from ossapi.ossapi import (OssapiV1, ReplayUnavailableException,
     InvalidKeyException, APIException)
-from ossapi.ossapiv2 import OssapiV2, Grant, Scope
+from ossapi.ossapiv2 import Ossapi, Grant, Scope
 from ossapi.models import (Beatmap, BeatmapCompact, BeatmapUserScore,
     ForumTopicAndPosts, Search, CommentBundle, Cursor, Score,
     BeatmapsetSearchResult, ModdingHistoryEventsBundle, User, Rankings,
     BeatmapScores, KudosuHistory, Beatmapset, BeatmapPlaycount, Spotlight,
     Spotlights, WikiPage, _Event, Event, BeatmapsetDiscussionPosts, Build,
-    ChangelogListing, MultiplayerScores, MultiplayerScoresCursor,
+    ChangelogListing, MultiplayerScores,
     BeatmapsetDiscussionVotes, CreatePMResponse, BeatmapsetDiscussions,
-    UserCompact, BeatmapsetCompact, ForumPoll)
+    UserCompact, BeatmapsetCompact, ForumPoll, Room, RoomPlaylistItem,
+    RoomPlaylistItemMod, RoomLeaderboardScore, RoomLeaderboardUserScore,
+    RoomLeaderboard, Match, Matches, MatchResponse, ScoreMatchInfo, MatchGame,
+    MatchEventDetail, MatchEvent, ScoringType, TeamType)
 from ossapi.enums import (GameMode, ScoreType, RankingFilter, RankingType,
     UserBeatmapType, BeatmapDiscussionPostSort, UserLookupKey,
     BeatmapsetEventType, CommentableType, CommentSort, ForumTopicSort,
@@ -21,11 +24,13 @@ from ossapi.enums import (GameMode, ScoreType, RankingFilter, RankingType,
     BeatmapsetDiscussionVoteSort, BeatmapsetStatus, MessageType,
     BeatmapsetSearchCategory, BeatmapsetSearchMode,
     BeatmapsetSearchExplicitContent, BeatmapsetSearchLanguage,
-    BeatmapsetSearchGenre, NewsPostKey, BeatmapsetSearchSort)
+    BeatmapsetSearchGenre, NewsPostKey, BeatmapsetSearchSort, RoomType,
+    RoomCategory, RoomSearchType, MatchEventType)
 from ossapi.mod import Mod
 from ossapi.replay import Replay
 from ossapi.version import __version__
 from ossapi.encoder import ModelEncoder, serialize_model
+from ossapi.ossapiv2_async import OssapiAsync
 
 from oauthlib.oauth2 import AccessDeniedError, TokenExpiredError
 from oauthlib.oauth2.rfc6749.errors import InsufficientScopeError
@@ -33,19 +38,23 @@ from oauthlib.oauth2.rfc6749.errors import InsufficientScopeError
 
 __all__ = [
     # OssapiV1
-    "Ossapi", "ReplayUnavailableException", "InvalidKeyException",
+    "OssapiV1", "ReplayUnavailableException", "InvalidKeyException",
     "APIException",
     # OssapiV2 core
-    "OssapiV2", "Grant", "Scope",
+    "Ossapi", "OssapiAsync", "Grant", "Scope",
     # OssapiV2 models
     "Beatmap", "BeatmapCompact", "BeatmapUserScore", "ForumTopicAndPosts",
     "Search", "CommentBundle", "Cursor", "Score", "BeatmapsetSearchResult",
     "ModdingHistoryEventsBundle", "User", "Rankings", "BeatmapScores",
     "KudosuHistory", "Beatmapset", "BeatmapPlaycount", "Spotlight",
     "Spotlights", "WikiPage", "_Event", "Event", "BeatmapsetDiscussionPosts",
-    "Build", "ChangelogListing", "MultiplayerScores", "MultiplayerScoresCursor",
+    "Build", "ChangelogListing", "MultiplayerScores",
     "BeatmapsetDiscussionVotes", "CreatePMResponse",
     "BeatmapsetDiscussions", "UserCompact", "BeatmapsetCompact", "ForumPoll",
+    "Room", "RoomPlaylistItem", "RoomPlaylistItemMod", "RoomLeaderboardScore",
+    "RoomLeaderboardUserScore", "RoomLeaderboard", "Match", "Matches",
+    "MatchResponse", "ScoreMatchInfo", "MatchGame", "MatchEventDetail",
+    "MatchEvent",
     # OssapiV2 enums
     "GameMode", "ScoreType", "RankingFilter", "RankingType",
     "UserBeatmapType", "BeatmapDiscussionPostSort", "UserLookupKey",
@@ -54,7 +63,9 @@ __all__ = [
     "BeatmapsetDiscussionVoteSort", "BeatmapsetStatus", "MessageType",
     "BeatmapsetSearchCategory", "BeatmapsetSearchMode",
     "BeatmapsetSearchExplicitContent", "BeatmapsetSearchLanguage",
-    "BeatmapsetSearchGenre", "NewsPostKey", "BeatmapsetSearchSort",
+    "BeatmapsetSearchGenre", "NewsPostKey", "BeatmapsetSearchSort", "RoomType",
+    "RoomCategory", "RoomSearchType", "MatchEventType", "ScoringType",
+    "TeamType",
     # OssapiV2 exceptions
     "AccessDeniedError", "TokenExpiredError", "InsufficientScopeError",
     # misc
