@@ -1,7 +1,8 @@
 # opt-in to forward type annotations
-# https://docs.python.org/3.7/whatsnew/3.7.html#pep-563-postponed-evaluation-of-annotations
+# https://docs.python.org/3.7/whatsnew/3.7.html#pep-563-postponed-evaluation-
+# of-annotations
 from __future__ import annotations
-from typing import Optional, TypeVar, Generic, Any, List, Union
+from typing import Optional, TypeVar, Generic, Any, List, Union, Dict
 from dataclasses import dataclass
 
 from ossapi.mod import Mod
@@ -14,7 +15,8 @@ from ossapi.enums import (UserAccountHistory, ProfileBanner, UserBadge, Country,
     EventBeatmap, BeatmapsetApproval, EventBeatmapset, KudosuVote,
     BeatmapsetEventType, UserRelationType, UserLevel, UserGradeCounts,
     GithubUser, ChangelogSearch, ForumTopicType, ForumPostBody, ForumTopicSort,
-    ChannelType, ReviewsConfig, NewsSearch, Nomination, RankHighest)
+    ChannelType, ReviewsConfig, NewsSearch, Nomination, RankHighest, RoomType,
+    RoomCategory, MatchEventType, ScoringType, TeamType)
 from ossapi.utils import Datetime, Model, BaseModel, Field
 
 T = TypeVar("T")
@@ -318,19 +320,24 @@ class Beatmapset(BeatmapsetCompact):
     submitted_date: Optional[Datetime]
     tags: str
     current_nominations: Optional[List[Nomination]]
+    deleted_at: Optional[Datetime]
 
     def expand(self) -> Beatmapset:
         return self
 
-
-class Match(Model):
-    pass
+# undocumented, but defined here to avoid a forward reference in Score.
+class ScoreMatchInfo(Model):
+    slot: int
+    team: str
+    pass_: bool = Field(name="pass")
 
 class Score(Model):
     """
     https://osu.ppy.sh/docs/index.html#score
     """
-    id: int
+    # can be null for match scores, eg the scores
+    # in https://osu.ppy.sh/community/matches/97947404
+    id: Optional[int]
     best_id: Optional[int]
     user_id: int
     accuracy: float
@@ -354,9 +361,17 @@ class Score(Model):
     rank_global: Optional[int]
     weight: Optional[Weight]
     _user: Optional[UserCompact] = Field(name="user")
-    match: Optional[Match]
+    match: Optional[ScoreMatchInfo]
     type: str
 
+    @classmethod
+    def preprocess_data(cls, data):
+        # scores from matches (api.match) return perfect as an int instead of a
+        # bool (same as api v1). Convert to a bool here.
+        if isinstance(data["perfect"], int):
+            data["perfect"] = bool(data["perfect"])
+        return data
+
     def user(self) -> Union[UserCompact, User]:
         return self._fk_user(self.user_id, existing=self._user)
 
@@ -746,7 +761,7 @@ class ChangelogListing(Model):
     streams: List[UpdateStream]
 
 class MultiplayerScores(Model):
-    cursor: MultiplayerScoresCursor
+    cursor_string: str
     params: str
     scores: List[MultiplayerScore]
     total: Optional[int]
@@ -775,10 +790,6 @@ class MultiplayerScoresAround(Model):
     higher: List[MultiplayerScore]
     lower: List[MultiplayerScore]
 
-class MultiplayerScoresCursor(Model):
-    score_id: int
-    total_score: int
-
 class NewsListing(Model):
     cursor: CursorT
     cursor_string: str
@@ -891,7 +902,8 @@ class BeatmapsetDiscussions(Model):
     beatmapsets: List[Beatmapset]
 
 class BeatmapsetDiscussionReview(Model):
-    # https://github.com/ppy/osu-web/blob/master/app/Libraries/BeatmapsetDiscussionReview.php
+    # https://github.com/ppy/osu-web/blob/master/app/Libraries/BeatmapsetDis
+    # cussionReview.php
     max_blocks: int
 
 class BeatmapsetDiscussionPosts(Model):
@@ -957,7 +969,10 @@ class BeatmapsetEventCommentWithSourceUser(BeatmapsetEventCommentNoPost):
 
 class BeatmapsetEvent(Model):
     # https://github.com/ppy/osu-web/blob/master/app/Models/BeatmapsetEvent.php
-    # https://github.com/ppy/osu-web/blob/master/app/Transformers/BeatmapsetEventTransformer.php
+    #
+    # https://github.com/ppy/osu-web/blob/master/app/Transformers/BeatmapsetEv
+    # entTransformer.php
+
     id: int
     type: BeatmapsetEventType
     comment: str
@@ -1013,7 +1028,7 @@ class ChatChannel(Model):
     channel_id: int
     description: Optional[str]
     icon: Optional[str]
-    # documented as non-optional (try pming tillerino with this non-optional)
+    # documented as non-optional (to see that it can be null, pm tillerino)
     moderated: Optional[bool]
     name: str
     type: ChannelType
@@ -1047,14 +1062,16 @@ class CreatePMResponse(Model):
     presence: Optional[List[ChatChannel]]
 
 class ModdingHistoryEventsBundle(Model):
-    # https://github.com/ppy/osu-web/blob/master/app/Libraries/ModdingHistoryEventsBundle.php#L84
+    # https://github.com/ppy/osu-web/blob/master/app/Libraries/ModdingHistor
+    # yEventsBundle.php#L84
     events: List[BeatmapsetEvent]
     reviewsConfig: BeatmapsetDiscussionReview
     users: List[UserCompact]
 
 class UserRelation(Model):
     # undocumented (and not a class on osu-web)
-    # https://github.com/ppy/osu-web/blob/master/app/Transformers/UserRelationTransformer.php#L16
+    # https://github.com/ppy/osu-web/blob/master/app/Transformers/UserRelatio
+    # nTransformer.php#L16
     target_id: int
     relation_type: UserRelationType
     mutual: bool
@@ -1087,11 +1104,116 @@ class UserStatistics(Model):
     variants: Optional[Any]
     global_rank_exp: Optional[float]
     pp_exp: float
+    count_100: int
+    count_300: int
+    count_50: int
+    count_miss: int
 
 class UserStatisticsRulesets(Model):
     # undocumented
-    # https://github.com/ppy/osu-web/blob/master/app/Transformers/UserStatisticsRulesetsTransformer.php
+    # https://github.com/ppy/osu-web/blob/master/app/Transformers/UserStatisti
+    # csRulesetsTransformer.php
     osu: Optional[UserStatistics]
     taiko: Optional[UserStatistics]
     fruits: Optional[UserStatistics]
     mania: Optional[UserStatistics]
+
+class RoomPlaylistItemMod(Model):
+    acronym: str
+    settings: Dict[str, Any]
+
+class RoomPlaylistItem(Model):
+    id: int
+    room_id: int
+    beatmap_id: int
+    ruleset_id: int
+    allowed_mods: List[RoomPlaylistItemMod]
+    required_mods: List[RoomPlaylistItemMod]
+    expired: bool
+    owner_id: int
+    # null for playlist items which haven't finished yet, I think
+    playlist_order: Optional[int]
+    # null for playlist items which haven't finished yet, I think
+    played_at: Optional[Datetime]
+    beatmap: BeatmapCompact
+
+class Room(Model):
+    id: int
+    name: str
+    category: RoomCategory
+    type: RoomType
+    user_id: int
+    starts_at: Datetime
+    ends_at: Datetime
+    max_attempts: Optional[int]
+    participant_count: int
+    channel_id: int
+    active: bool
+    has_password: bool
+    queue_mode: str
+    auto_skip: bool
+    host: UserCompact
+    playlist: List[RoomPlaylistItem]
+    recent_participants: List[UserCompact]
+
+class RoomLeaderboardScore(Model):
+    accuracy: float
+    attempts: int
+    completed: int
+    pp: float
+    room_id: int
+    total_score: int
+    user_id: int
+    user: UserCompact
+
+class RoomLeaderboardUserScore(RoomLeaderboardScore):
+    position: int
+
+class RoomLeaderboard(Model):
+    leaderboard: List[RoomLeaderboardScore]
+    user_score: RoomLeaderboardUserScore
+
+class Match(Model):
+    id: int
+    start_time: Datetime
+    # null for matches which haven't finished yet, I think
+    end_time: Optional[Datetime]
+    name: str
+
+class Matches(Model):
+    matches: List[Match]
+    cursor: CursorT
+    params: Any
+
+class MatchGame(Model):
+    id: int
+    start_time: Datetime
+    end_time: Datetime
+    mode: GameMode
+    mode_int: int
+    scoring_type: ScoringType
+    team_type: TeamType
+    mods: List[Mod]
+    beatmap: BeatmapCompact
+    scores: List[Score]
+
+class MatchEventDetail(Model):
+    type: MatchEventType
+    # seems to only be used for MatchEventType.OTHER
+    text: Optional[str]
+
+class MatchEvent(Model):
+    id: int
+    detail: MatchEventDetail
+    timestamp: Datetime
+    # can be none for MatchEventType.OTHER
+    user_id: Optional[int]
+    game: Optional[MatchGame]
+
+class MatchResponse(Model):
+    match: Match
+    events: List[MatchEvent]
+    users: List[UserCompact]
+    first_event_id: int
+    latest_event_id: int
+    current_game_id: Optional[int]
